from typing import List


# return string like '[aaa] -> [bbb] -> [ccc]'
def format_exception_chain(e: BaseException):
    # recursive function, get exception chain from __cause__
    def get_exception_chain(e: BaseException) -> List[BaseException]:
        return [e] if e.__cause__ is None else [e] + get_exception_chain(e.__cause__)

    return ''.join(f'[{exc}]' if i == 0 else f' -> [{exc}]' for i, exc in enumerate(reversed(get_exception_chain(e))))


def test():
    try:
        try:
            try:
                try:
                    raise Exception('aaa') from None
                except Exception as e:
                    raise Exception('bbb') from e
            except Exception as e:
                raise Exception('ccc') from e
        except Exception as e:
            raise Exception('ddd') from e
    except Exception as e:
        print(f'出错：{format_exception_chain(e)}')


if __name__ == '__main__':
    test()



import os
import psutil
import functools
import cProfile
from timeit import default_timer as timer

def second_to_human_friendly_time(seconds: float, precision: int=3) -> str:
    precision = 3 if precision < 1 or precision > 6 else precision
    days = int(seconds // (60*60*24))
    seconds %= 60*60*24
    hours = int(seconds // (60*60))
    seconds %= 60*60
    minutes = int(seconds // 60)
    seconds %= 60
    print(f'{days}d {hours:02}:{minutes:02}:{seconds:0>{precision+3}.{precision}f}')
    if days > 0:
        return f'{days}d {hours:02}:{minutes:02}:{seconds:0>{precision+3}.{precision}f}'
    elif hours > 0:
        return f'{hours:02}:{minutes:02}:{seconds:0>{precision+3}.{precision}f}'
    elif minutes > 0:
        return f'{minutes:02}:{seconds:0>{precision+3}.{precision}f}'
    else:
        return f'{seconds:.{precision}f}s'


def measure_time(func):
    @functools.wraps(func)
    def time_it(*args, **kwargs):
        start = timer()
        try:
            return func(*args, **kwargs)
        finally:
            end = timer()
            print(f'Time Usage for {func.__name__}(): {end - start:.8f} s')

    return time_it


def measure_profile(func):
    @functools.wraps(func)
    def profile_it(*args, **kwargs):
        pr = cProfile.Profile()
        pr.enable()
        try:
            return func(*args, **kwargs)
        finally:
            pr.disable()
            print(f'[cProfile]:')
            pr.print_stats()

    return profile_it


def byte_to_human_friendly_size(bytes_number: int, precision: int=2) -> str:
    if bytes_number >= 1024**3:
        return f'{bytes_number/(1024**3):.{precision}f}GB'
    elif bytes_number >= 1024**2:
        return f'{bytes_number/(1024**2):.{precision}f}MB'
    elif bytes_number >= 1024**1:
        return f'{bytes_number/(1024**1):.{precision}f}KB'
    else:
        return f'{bytes_number}B'


def measure_memory(func):
    @functools.wraps(func)
    def check_memory(*args, **kwargs):
        process = psutil.Process(os.getpid())
        start = process.memory_info().rss
        try:
            return func(*args, **kwargs)
        finally:
            end = process.memory_info().rss
            print(f'Memory Usage: {byte_to_human_friendly_size(end - start)}')

    return check_memory

@measure_profile
def test():
    print(byte_to_human_friendly_size(1234567))
    print(second_to_human_friendly_time(1234.412))

if __name__ == '__main__':
    test()



from pathlib import Path
from os import scandir, DirEntry, PathLike
from typing import Iterator, Union, AnyStr

# Return all file or symlink Path obj in `path`
def scan_dir(path: Union[AnyStr, PathLike]) -> Iterator[Path]:
    for entry in scan_dir_raw(path):
        yield Path(entry.path)


# Return all file or symlink os.DirEntry obj in `path`
def scan_dir_raw(path: Union[AnyStr, PathLike]) -> Iterator[DirEntry]:
    with scandir(path) as it:
        for entry in it:
            if entry.is_dir():
                for e in scan_dir_raw(Path(entry.path)):
                    yield e
            else:
                yield entry



import setuptools
from knives import __version__ as knives_version

with open('Readme.md', 'r', encoding='utf-8') as fh:
    long_description = fh.read()

setuptools.setup(
    name='knives',
    version=knives_version,
    author='owtotwo',
    author_email='owtotwo@163.com',
    description='A commonly used python tools library for myself',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/owtotwo/knives',
    py_modules=['knives'],
    packages=setuptools.find_packages(),
    classifiers=[
        'Programming Language :: Python :: 3 :: Only',
        'License :: OSI Approved :: GNU Lesser General Public License v3 (LGPLv3)',
        'Operating System :: Microsoft :: Windows :: Windows 10',
    ],
    python_requires='>=3.7',
)



from typing import Final, Sequence

from knives.exception import format_exception_chain
from knives.measure import measure_time, measure_profile, measure_memory
from knives.pathlib import scan_dir, scan_dir_raw

__all__ = ['format_exception_chain', 'measure_time', 'measure_profile', 'measure_memory', 'scan_dir', 'scan_dir_raw']

__author__: Final[str] = "owtotwo"
__copyright__: Final[str] = "Copyright 2020 owtotwo"
__credits__: Final[Sequence[str]] = ["owtotwo"]
__license__: Final[str] = "LGPLv3"
__version__: Final[str] = "0.0.3"
__maintainer__: Final[str] = "owtotwo"
__email__: Final[str] = "owtotwo@163.com"
__status__: Final[str] = "Experimental"
